@@ -3,7 +3,7 @@
 import logging
 from os import remove
 from os.path import join
-from typing import Optional, List, Tuple, Dict, Union
+from typing import Optional, List, Tuple, Dict, Union, Any
 
 from hdx.utilities import raisefrom, is_valid_uuid
 from hdx.utilities.downloader import Download
@@ -132,7 +132,7 @@ class Resource(HDXObject):
 
     def set_file_to_upload(self, file_to_upload):
         # type: (str) -> None
-        """Set the file uploaded to the local path provided
+        """Delete any existing url and set the file uploaded to the local path provided
 
         Args:
             file_to_upload (str): Local path to file to upload
@@ -140,16 +140,17 @@ class Resource(HDXObject):
         Returns:
             None
         """
+        if 'url' in self.data:
+            del self.data['url']
         self.file_to_upload = file_to_upload
 
     def check_url_filetoupload(self):
-        # type: () -> bool
-        """Check if url or file to upload provided for resource and add resource_type and url_type if not supplied.
+        # type: () -> None
+        """Check if url or file to upload provided for resource and add resource_type and url_type if not supplied
 
         Returns:
-            bool: Whether url field and file_to_upload are both populated
+            None
         """
-        url_file_to_upload = False
         if self.file_to_upload is None:
             if 'url' in self.data:
                 if 'resource_type' not in self.data:
@@ -160,14 +161,14 @@ class Resource(HDXObject):
                 raise HDXError('Either a url or a file to upload must be supplied!')
         else:
             if 'url' in self.data:
-                url_file_to_upload = True
+                if self.data['url'] != hdx.data.dataset.Dataset.temporary_url:
+                    raise HDXError('Either a url or a file to upload must be supplied not both!')
             if 'resource_type' not in self.data:
                 self.data['resource_type'] = 'file.upload'
             if 'url_type' not in self.data:
                 self.data['url_type'] = 'upload'
             if 'tracking_summary' in self.data:
                 del self.data['tracking_summary']
-        return url_file_to_upload
 
     def check_required_fields(self, ignore_fields=list()):
         # type: (List[str]) -> None
@@ -183,30 +184,21 @@ class Resource(HDXObject):
         self.check_url_filetoupload()
         self._check_required_fields('resource', ignore_fields)
 
-    def _remove_url(self, url_file_to_upload):
-        # type: (bool) -> None
-        """If input parameter is True, remove url from data.
-
-        Args:
-            url_file_to_upload (bool): Whether to remove url
-
-        Returns:
-            None
-        """
-        if url_file_to_upload:
-            logger.warning(
-                'Either a url or a file to upload must be supplied not both! Deleting url from resource %s!' % self.data['id'])
-            del self.data['url']
-
-    def update_in_hdx(self):
-        # type: () -> None
+    def update_in_hdx(self, **kwargs):
+        # type: (Any) -> None
         """Check if resource exists in HDX and if so, update it
 
+        Args:
+            **kwargs: See below
+            operation (string): Operation to perform eg. patch. Defaults to update.
+
         Returns:
             None
         """
-        self._remove_url(self.check_url_filetoupload())
-        self._update_in_hdx('resource', 'id', self.file_to_upload)
+        self._check_load_existing_object('resource', 'id')
+        if self.file_to_upload and 'url' in self.data:
+            del self.data['url']
+        self._merge_hdx_update('resource', 'id', self.file_to_upload, **kwargs)
 
     def create_in_hdx(self):
         # type: () -> None
@@ -215,16 +207,14 @@ class Resource(HDXObject):
         Returns:
             None
         """
-        url_file_to_upload = self.check_url_filetoupload()
+        self.check_required_fields()
         id = self.data.get('id')
         if id and self._load_from_hdx('resource', id):
             logger.warning('%s exists. Updating %s' % ('resource', id))
-            self._remove_url(url_file_to_upload)
+            if self.file_to_upload and 'url' in self.data:
+                del self.data['url']
             self._merge_hdx_update('resource', 'id', self.file_to_upload)
         else:
-            if url_file_to_upload:
-                raise HDXError('Either a url or a file to upload must be supplied not both!')
-            self._check_required_fields('resource', list())
             self._save_to_hdx('create', 'name', self.file_to_upload)
 
     def delete_from_hdx(self):
@@ -297,7 +287,7 @@ class Resource(HDXObject):
         format = '.%s' % self.data['format']
         if format not in filename:
             filename = '%s%s' % (filename, format)
-        with Download() as downloader:
+        with Download(full_agent=self.configuration.get_user_agent()) as downloader:
             path = downloader.download_file(url, folder, filename)
             return url, path
 
@@ -390,7 +380,7 @@ class Resource(HDXObject):
                     row[i] = str(val)
                 yield (number, headers, row)
 
-        with Download() as downloader:
+        with Download(full_agent=self.configuration.get_user_agent()) as downloader:
             try:
                 stream = downloader.get_tabular_stream(path, headers=1, post_parse=[convert_to_text],
                                                        bytes_sample_size=1000000)
@@ -569,15 +559,6 @@ class Resource(HDXObject):
         """
         self.create_datastore_for_topline(2, path=path)
 
-    def touch(self):
-        # type: () -> None
-        """Touch resource
-
-        Returns:
-            None
-        """
-        self._write_to_hdx('patch', self.data, 'id')
-
     def get_resource_views(self):
         # type: () -> List[ResourceView]
         """Get any resource views in the resource
