@@ -4,9 +4,12 @@ import os
 
 import requests
 import six
-
+from hdx.utilities.downloader import Download
 from hdx.utilities.email import Email
 from hdx.utilities.session import get_session
+from hdx.utilities.useragent import UserAgent, UserAgentError
+
+from hdx.version import get_api_version
 
 if six.PY2:
     from UserDict import IterableUserDict as UserDict
@@ -17,7 +20,7 @@ else:
 import logging
 from base64 import b64decode
 from os.path import expanduser, join, isfile
-from typing import Optional, Any, Dict
+from typing import Optional, Any, Dict, Tuple
 
 import ckanapi
 
@@ -37,9 +40,13 @@ class Configuration(UserDict, object):
 
     Args:
         **kwargs: See below
-        hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
+        user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+        user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+        user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
+        hdx_url (str): HDX url to use. Overrides hdx_site.
+        hdx_site (str): HDX site to use eg. prod, test.
         hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
-        hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
+        hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
         hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
         hdx_config_json (str): Path to JSON HDX configuration OR
         hdx_config_yaml (str): Path to YAML HDX configuration
@@ -55,7 +62,9 @@ class Configuration(UserDict, object):
     home_folder = expanduser('~')
     default_hdx_base_config_yaml = script_dir_plus_file('hdx_base_configuration.yml', ConfigurationError)
     default_hdx_config_yaml = join(home_folder, '.hdx_configuration.yml')
-    default_user_agent_config_yaml = join(home_folder, '.useragent.yml')
+
+    apiversion = get_api_version()
+    prefix = 'HDXPythonLibrary/%s' % apiversion
 
     def __init__(self, **kwargs):
         # type: (Any) -> None
@@ -149,19 +158,27 @@ class Configuration(UserDict, object):
 
         self.data = merge_two_dictionaries(hdx_base_config_dict, project_config_dict)
 
+        ua = kwargs.get('full_agent')
+        if ua:
+            self.user_agent = ua
+        else:
+            try:
+                self.user_agent = UserAgent.get(prefix=Configuration.prefix, **kwargs)
+            except UserAgentError:
+                self.user_agent = UserAgent.get(prefix=Configuration.prefix, **self.data)
         self.hdx_read_only = kwargs.get('hdx_read_only', self.data.get('hdx_read_only', False))
         logger.info('Read only access to HDX: %s' % str(self.hdx_read_only))
         self.hdx_key = kwargs.get('hdx_key', self.data.get('hdx_key'))
         if not self.hdx_key and not self.hdx_read_only:
             raise ConfigurationError('No HDX API key supplied as a parameter or in configuration!')
-        self.hdx_site = 'hdx_%s_site' % kwargs.get('hdx_site', self.data.get('hdx_site', 'test'))
-        if self.hdx_site not in self.data:
-            raise ConfigurationError('%s not defined in configuration!' % self.hdx_site)
-
-    @staticmethod
-    def get_version():
-        version_file = open(script_dir_plus_file('version.txt', Configuration))
-        return version_file.read().strip()
+        hdx_url = kwargs.get('hdx_url', self.data.get('hdx_url'))
+        if hdx_url:
+            self.hdx_site = 'hdx_custom_site'
+            self.data[self.hdx_site] = {'url': hdx_url}
+        else:
+            self.hdx_site = 'hdx_%s_site' % kwargs.get('hdx_site', self.data.get('hdx_site', 'test'))
+            if self.hdx_site not in self.data:
+                raise ConfigurationError('%s not defined in configuration!' % self.hdx_site)
 
     def set_read_only(self, read_only=True):
         # type: (bool) -> None
@@ -202,6 +219,17 @@ class Configuration(UserDict, object):
             return None
         return self.hdx_key
 
+    def get_user_agent(self):
+        # type: () -> str
+        """
+        Return user agent
+
+        Returns:
+            str: User agent
+
+        """
+        return self.user_agent
+
     def get_hdx_site_url(self):
         # type: () -> str
         """
@@ -214,20 +242,20 @@ class Configuration(UserDict, object):
         return self.data[self.hdx_site]['url']
 
     def _get_credentials(self):
-        # type: () -> tuple
+        # type: () -> Optional[Tuple[str, str]]
         """
         Return HDX site username and password
 
         Returns:
-            tuple: HDX site username and password
+            Optional[Tuple[str, str]]: HDX site username and password or None
 
         """
         site = self.data[self.hdx_site]
-        username = site['username']
+        username = site.get('username')
         if username:
             return b64decode(username).decode('utf-8'), b64decode(site['password']).decode('utf-8')
         else:
-            return '', ''
+            return None
 
     def remoteckan(self):
         # type: () -> ckanapi.RemoteCKAN
@@ -242,8 +270,21 @@ class Configuration(UserDict, object):
             raise ConfigurationError('There is no remote CKAN set up! Use Configuration.create(**kwargs)')
         return self._remoteckan
 
+    def downloader(self):
+        # type: () -> Download
+        """
+        Return the Download object (see HDX Python Utilities library)
+
+        Returns:
+            Download: The Download object
+
+        """
+        if self._downloader is None:
+            raise ConfigurationError('There is no Download object set up! Use Configuration.create(**kwargs)')
+        return self._downloader
+
     def call_remoteckan(self, *args, **kwargs):
-        # type: (Any) -> Dict
+        # type: (Any, Any) -> Dict
         """
         Calls the remote CKAN
 
@@ -256,16 +297,46 @@ class Configuration(UserDict, object):
 
         """
         requests_kwargs = kwargs.get('requests_kwargs', dict())
-        requests_kwargs['auth'] = self._get_credentials()
+        credentials = self._get_credentials()
+        if credentials:
+            requests_kwargs['auth'] = credentials
         kwargs['requests_kwargs'] = requests_kwargs
         apikey = kwargs.get('apikey', self.get_api_key())
         kwargs['apikey'] = apikey
         return self.remoteckan().call_action(*args, **kwargs)
 
+    @staticmethod
+    def _environment_variables(**kwargs):
+        # type: (Any) -> Any
+        """
+        Overwrite keyword arguments with environment variables
+
+        Args:
+            **kwargs: See below
+            hdx_url (str): HDX url to use. Overrides hdx_site.
+            hdx_site (str): HDX site to use eg. prod, test. Defaults to test.
+            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
+
+        Returns:
+            kwargs: Changed keyword arguments
+
+        """
+
+        hdx_key = os.getenv('HDX_KEY')
+        if hdx_key is not None:
+            kwargs['hdx_key'] = hdx_key
+        hdx_url = os.getenv('HDX_URL')
+        if hdx_url is not None:
+            kwargs['hdx_url'] = hdx_url
+        else:
+            hdx_site = os.getenv('HDX_SITE')
+            if hdx_site is not None:
+                kwargs['hdx_site'] = hdx_site
+        return kwargs
+
     @classmethod
     def create_remoteckan(cls, site_url, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
-                          session=get_session(method_whitelist=frozenset(['HEAD', 'TRACE', 'GET', 'POST', 'PUT',
-                                                                          'OPTIONS', 'DELETE'])), **kwargs):
+                          session=None, **kwargs):
         # type: (str, Optional[str], Optional[str], Optional[str], requests.Session, Any) -> ckanapi.RemoteCKAN
         """
         Create remote CKAN instance from configuration
@@ -281,23 +352,24 @@ class Configuration(UserDict, object):
             ckanapi.RemoteCKAN: Remote CKAN instance
 
         """
-        prefix = 'HDXPythonLibrary/%s' % Configuration.get_version()
-        if not user_agent:
-            ua = Configuration.load_user_agent(prefix, user_agent_config_yaml, user_agent_lookup)
+        if not session:
+            session = get_session(user_agent, user_agent_config_yaml, user_agent_lookup, prefix=Configuration.prefix,
+                                  method_whitelist=frozenset(['HEAD', 'TRACE', 'GET', 'POST', 'PUT',
+                                                              'OPTIONS', 'DELETE']), **kwargs)
+            ua = session.headers['User-Agent']
         else:
-            ua = cls._construct_user_agent(kwargs, prefix, user_agent)
+            ua = kwargs.get('full_agent')
+            if not ua:
+                ua = UserAgent.get(user_agent, user_agent_config_yaml, user_agent_lookup, prefix=Configuration.prefix,
+                                   **kwargs)
         return ckanapi.RemoteCKAN(site_url, user_agent=ua, session=session)
 
-    def setup_remoteckan(self, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
-                         remoteckan=None, **kwargs):
-        # type: (Optional[str], Optional[str], Optional[str], Optional[ckanapi.RemoteCKAN], Any) -> None
+    def setup_remoteckan(self, remoteckan=None, **kwargs):
+        # type: (Optional[ckanapi.RemoteCKAN], Any) -> None
         """
         Set up remote CKAN from provided CKAN or by creating from configuration
 
         Args:
-            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
-            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
-            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
             remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
 
         Returns:
@@ -305,8 +377,8 @@ class Configuration(UserDict, object):
 
         """
         if remoteckan is None:
-            self._remoteckan = self.create_remoteckan(self.get_hdx_site_url(), user_agent, user_agent_config_yaml,
-                                                      user_agent_lookup, **kwargs)
+            self._remoteckan = self.create_remoteckan(self.get_hdx_site_url(), full_agent=self.get_user_agent(),
+                                                      **kwargs)
         else:
             self._remoteckan = remoteckan
 
@@ -346,62 +418,6 @@ class Configuration(UserDict, object):
         """
         self._emailer = Email(**kwargs)
 
-    @staticmethod
-    def _construct_user_agent(configdict, prefix, ua):
-        # type: (Dict, str, str) -> str
-        """
-        Construct user agent
-
-        Args:
-            configdict (str): Additional configuration for user agent
-            prefix (str): Text to put at start of user agent
-            ua (str): Custom user agent text
-
-        Returns:
-            str: Full user agent string
-
-        """
-        if not ua:
-            raise ConfigurationError("user_agent should be supplied in a YAML config file or directly as a parameter. It can be your project's name for example.")
-        preprefix = configdict.get('preprefix')
-        if preprefix:
-            user_agent = '%s:' % preprefix
-        else:
-            user_agent = ''
-        if prefix:
-            user_agent = '%s%s-' % (user_agent, prefix)
-        user_agent = '%s%s' % (user_agent, ua)
-        return user_agent
-
-    @classmethod
-    def load_user_agent(cls, prefix, user_agent_config_yaml, user_agent_lookup=None):
-        # type: (str, str, Optional[str]) -> str
-        """
-        Load user agent YAML file
-
-        Args:
-            prefix (str): Text to put at start of user agent
-            user_agent_config_yaml (str): Path to user agent YAML file
-            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
-
-        Returns:
-            str: user agent
-
-        """
-        if not user_agent_config_yaml:
-            user_agent_config_yaml = Configuration.default_user_agent_config_yaml
-            logger.info('No user agent or user agent config file given. Using default user agent config file: %s.' % user_agent_config_yaml)
-        if not isfile(user_agent_config_yaml):
-            raise ConfigurationError("User_agent should be supplied in a YAML config file or directly as a parameter. It can be your project's name for example.")
-        logger.info('Loading user agent config from: %s' % user_agent_config_yaml)
-        user_agent_config_dict = load_yaml(user_agent_config_yaml)
-        if user_agent_lookup:
-            user_agent_config_dict = user_agent_config_dict.get(user_agent_lookup)
-        if not user_agent_config_dict:
-            raise ConfigurationError("No user agent information read from: %s" % user_agent_config_yaml)
-        ua = user_agent_config_dict.get('user_agent')
-        return cls._construct_user_agent(user_agent_config_dict, prefix, ua)
-
     @classmethod
     def read(cls):
         # type: () -> 'Configuration'
@@ -425,9 +441,13 @@ class Configuration(UserDict, object):
         Args:
             configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
             **kwargs: See below
-            hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
+            user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+            user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
+            hdx_url (str): HDX url to use. Overrides hdx_site.
+            hdx_site (str): HDX site to use eg. prod, test.
             hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
-            hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
+            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
             hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
             hdx_config_json (str): Path to JSON HDX configuration OR
             hdx_config_yaml (str): Path to YAML HDX configuration
@@ -447,54 +467,23 @@ class Configuration(UserDict, object):
         else:
             cls._configuration = configuration
 
-    @staticmethod
-    def _environment_variables(**kwargs):
-        # type: (Any) -> Any
-        """
-        Overwrite keyword arguments with environment variables
-
-        Args:
-            **kwargs: See below
-            hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
-            hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
-            user_agent (Optional[str]): User agent string.
-
-        Returns:
-            kwargs: Changed keyword arguments
-
-        """
-
-        hdx_key = os.getenv('HDX_KEY')
-        if hdx_key is not None:
-            kwargs['hdx_key'] = hdx_key
-        user_agent = os.getenv('USER_AGENT')
-        if user_agent is not None:
-            kwargs['user_agent'] = user_agent
-        preprefix = os.getenv('PREPREFIX')
-        if preprefix is not None:
-            kwargs['preprefix'] = preprefix
-        hdx_site = os.getenv('HDX_SITE')
-        if hdx_site is not None:
-            kwargs['hdx_site'] = hdx_site
-        return kwargs
-
     @classmethod
-    def _create(cls, configuration=None, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
-                remoteckan=None, **kwargs):
-        # type: (Optional['Configuration'], Optional[str], Optional[str], Optional[str], Optional[ckanapi.RemoteCKAN], Any) -> str
+    def _create(cls, configuration=None, remoteckan=None, **kwargs):
+        # type: (Optional['Configuration'], Optional[ckanapi.RemoteCKAN], Any) -> str
         """
         Create HDX configuration
 
         Args:
             configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
-            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
-            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
-            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
             remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
             **kwargs: See below
-            hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
+            user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+            user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
+            hdx_url (str): HDX url to use. Overrides hdx_site.
+            hdx_site (str): HDX site to use eg. prod, test.
             hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
-            hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
+            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
             hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
             hdx_config_json (str): Path to JSON HDX configuration OR
             hdx_config_yaml (str): Path to YAML HDX configuration
@@ -510,30 +499,27 @@ class Configuration(UserDict, object):
 
         """
         kwargs = cls._environment_variables(**kwargs)
-        if 'user_agent' in kwargs:
-            user_agent = kwargs['user_agent']
-            del kwargs['user_agent']
         cls.setup(configuration, **kwargs)
-        cls._configuration.setup_remoteckan(user_agent, user_agent_config_yaml, user_agent_lookup, remoteckan, **kwargs)
+        cls._configuration.setup_remoteckan(remoteckan, **kwargs)
         return cls._configuration.get_hdx_site_url()
 
     @classmethod
-    def create(cls, configuration=None, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
-               remoteckan=None, **kwargs):
-        # type: (Optional['Configuration'], Optional[str], Optional[str], Optional[str], Optional[ckanapi.RemoteCKAN], Any) -> str
+    def create(cls, configuration=None, remoteckan=None, **kwargs):
+        # type: (Optional['Configuration'], Optional[ckanapi.RemoteCKAN], Any) -> str
         """
         Create HDX configuration. Can only be called once (will raise an error if called more than once).
 
         Args:
             configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
-            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
-            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
-            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
             remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
             **kwargs: See below
+            user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+            user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
+            hdx_url (str): HDX url to use. Overrides hdx_site.
             hdx_site (str): HDX site to use eg. prod, test.
             hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
-            hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
+            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
             hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
             hdx_config_json (str): Path to JSON HDX configuration OR
             hdx_config_yaml (str): Path to YAML HDX configuration
@@ -550,9 +536,7 @@ class Configuration(UserDict, object):
         """
         if cls._configuration is not None:
             raise ConfigurationError('Configuration already created!')
-        return cls._create(configuration=configuration, user_agent=user_agent,
-                           user_agent_config_yaml=user_agent_config_yaml, user_agent_lookup=user_agent_lookup,
-                           remoteckan=remoteckan, **kwargs)
+        return cls._create(configuration=configuration, remoteckan=remoteckan, **kwargs)
 
     @classmethod
     def delete(cls):
