@@ -22,9 +22,9 @@ from hdx.hdx_configuration import Configuration
 from hdx.hdx_tagscleanup import Tags, ChainRuleError
 from . import MockResponse, user_data, organization_data
 from .test_organization import organization_mockshow
+from .test_resource_view import resource_view_list
 from .test_showcase import showcase_resultdict
 from .test_user import user_mockshow
-from .test_resource_view import resource_view_list
 
 resulttags = [{'state': 'active', 'display_name': 'conflict', 'vocabulary_id': None,
                'id': '1dae41e5-eacd-4fa5-91df-8d80cf579e53', 'name': 'conflict'},
@@ -100,6 +100,7 @@ hxlupdate_list = [{'title': 'Quick Charts', 'resource_id': 'de6549d8-268b-4dfe-a
                    'package_id': '6f36a41c-f126-4b18-aaaf-6c2ddfbc5d4d', 'view_type': 'hdx_hxl_preview', 
                    'description': '', 'id': '29cc5894-4306-4bef-96ce-b7a833e7986a'}]
 
+
 def mockshow(url, datadict):
     if 'show' not in url:
         return MockResponse(404,
@@ -255,14 +256,14 @@ class TestDataset:
                   'id': 'aaafc63b-2234-48e3-8ccc-198d7cf0f3f3', 'name': 'political violence'}],
     }
 
-    resources_data = [{"id": "de6549d8-268b-4dfe-adaf-a4ae5c8510d5", "description": "Resource1",
-                       "name": "Resource1",
-                       "url": "http://resource1.xlsx",
-                       "format": "xlsx"},
-                      {"id": "DEF", "description": "Resource2",
-                       "name": "Resource2",
-                       "url": "http://resource2.csv",
-                       "format": "csv"}]
+    resources_data = [{'id': 'de6549d8-268b-4dfe-adaf-a4ae5c8510d5', 'description': 'Resource1',
+                       'name': 'Resource1',
+                       'url': 'http://resource1.xlsx',
+                       'format': 'xlsx'},
+                      {'id': 'DEF', 'description': 'Resource2',
+                       'name': 'Resource2',
+                       'url': 'http://resource2.csv',
+                       'format': 'csv'}]
 
     association = None
 
@@ -278,7 +279,7 @@ class TestDataset:
     def read(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return mockshow(url, datadict)
 
@@ -288,7 +289,7 @@ class TestDataset:
     def post_create(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 if isinstance(data, dict):
                     datadict = {k.decode('utf8'): v.decode('utf8') for k, v in data.items()}
                 else:
@@ -328,7 +329,7 @@ class TestDataset:
     def post_update(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 if isinstance(data, dict):
                     datadict = {k.decode('utf8'): v.decode('utf8') for k, v in data.items()}
                 else:
@@ -384,7 +385,7 @@ class TestDataset:
     def post_reorder(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 decodedata = data.decode('utf-8')
                 datadict = json.loads(decodedata)
                 if 'show' in url:
@@ -407,7 +408,7 @@ class TestDataset:
     def post_delete(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 decodedata = data.decode('utf-8')
                 datadict = json.loads(decodedata)
                 if 'show' in url:
@@ -432,7 +433,7 @@ class TestDataset:
     def search(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return mocksearch(url, datadict)
 
@@ -442,7 +443,7 @@ class TestDataset:
     def post_list(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return mocklist(url)
 
@@ -452,7 +453,7 @@ class TestDataset:
     def all(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return mockall(url, datadict)
 
@@ -462,7 +463,7 @@ class TestDataset:
     def user_read(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return user_mockshow(url, datadict)
 
@@ -472,7 +473,7 @@ class TestDataset:
     def organization_read(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return organization_mockshow(url, datadict)
 
@@ -482,7 +483,7 @@ class TestDataset:
     def showcase_read(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 if 'showcase_list' in url:
                     result = json.dumps([showcase_resultdict])
@@ -546,8 +547,8 @@ class TestDataset:
         with pytest.raises(HDXError):
             dataset.create_in_hdx()
 
-        config = Configuration(hdx_read_only=True)
-        config.setup_remoteckan(user_agent='test')
+        config = Configuration(user_agent='test', hdx_read_only=True)
+        config.setup_remoteckan()
         config.remoteckan().session = Configuration.read().remoteckan().session
         uniqueval = 'myconfig'
         config.unique = uniqueval
@@ -644,6 +645,7 @@ class TestDataset:
         assert len(dataset.resources) == 2
         resource['name'] = '123'
         resource.set_file_to_upload(None)
+        resource['url'] = 'http://lala'
         dataset.add_update_resource(resource)
         dataset.update_in_hdx()
         assert len(dataset.resources) == 3
