@@ -112,7 +112,7 @@ class TestUser:
     def read(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return user_mockshow(url, datadict)
 
@@ -122,7 +122,7 @@ class TestUser:
     def post_create(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 if 'show' in url:
                     return user_mockshow(url, datadict)
@@ -150,7 +150,7 @@ class TestUser:
     def post_update(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 if 'show' in url:
                     return user_mockshow(url, datadict)
@@ -180,7 +180,7 @@ class TestUser:
     def post_delete(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 decodedata = data.decode('utf-8')
                 datadict = json.loads(decodedata)
                 if 'show' in url:
@@ -201,7 +201,7 @@ class TestUser:
     def post_list(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 datadict = json.loads(data.decode('utf-8'))
                 return mocklist(url)
 
@@ -211,7 +211,7 @@ class TestUser:
     def post_listorgs(self):
         class MockSession(object):
             @staticmethod
-            def post(url, data, headers, files, allow_redirects, auth):
+            def post(url, data, headers, files, allow_redirects, auth=None):
                 decodedata = data.decode('utf-8')
                 datadict = json.loads(decodedata)
                 if 'user' in url:
