@@ -5,8 +5,10 @@ from os.path import join
 import ckanapi
 import pytest
 from hdx.utilities.loader import LoadError
+from hdx.utilities.useragent import UserAgentError
 
 from hdx.hdx_configuration import Configuration, ConfigurationError
+from hdx.version import get_api_version
 
 
 class TestConfiguration:
@@ -53,12 +55,12 @@ class TestConfiguration:
     def test_init(self, hdx_config_json, hdx_config_yaml, hdx_base_config_yaml, hdx_base_config_json, hdx_missing_site_config_json, project_config_json, project_config_yaml):
         default_config_file = Configuration.default_hdx_config_yaml
         Configuration.default_hdx_config_yaml = 'NOT EXIST'
-        with pytest.raises(ConfigurationError):
+        with pytest.raises(UserAgentError):
             Configuration()
         Configuration.default_hdx_config_yaml = default_config_file
 
         Configuration.default_hdx_config_yaml = hdx_config_yaml
-        assert Configuration().get_api_key() == '12345'
+        assert Configuration(user_agent='test').get_api_key() == '12345'
         Configuration.default_hdx_config_yaml = default_config_file
 
         with pytest.raises(IOError):
@@ -103,21 +105,23 @@ class TestConfiguration:
             Configuration(hdx_config_yaml=hdx_config_yaml, project_config_json='NOT_EXIST')
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_config_json=hdx_missing_site_config_json, project_config_json=project_config_json, hdx_base_config_json=hdx_base_config_json)
+            Configuration(user_agent='test', hdx_config_json=hdx_missing_site_config_json,
+                          project_config_json=project_config_json, hdx_base_config_json=hdx_base_config_json)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_site='NOT_EXIST', hdx_config_yaml=hdx_config_yaml, project_config_yaml=project_config_yaml)
-
-        with pytest.raises(ConfigurationError):
-            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_json=project_config_json,
+            Configuration(user_agent='test', hdx_site='NOT_EXIST', hdx_config_yaml=hdx_config_yaml,
                           project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
+            Configuration(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_json=project_config_json,
                           project_config_yaml=project_config_yaml)
 
         with pytest.raises(ConfigurationError):
-            Configuration(hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
+            Configuration(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
+                          project_config_yaml=project_config_yaml)
+
+        with pytest.raises(ConfigurationError):
+            Configuration(user_agent='test', hdx_config_yaml=hdx_config_yaml, project_config_dict={'la': 'la'},
                           project_config_json=project_config_json)
 
     def test_hdx_configuration_dict(self, project_config_yaml, mocksmtp):
@@ -125,8 +129,6 @@ class TestConfiguration:
                               hdx_base_config_dict={
                                                  'hdx_prod_site': {
                                                      'url': 'https://data.humdata.org/',
-                                                     'username': None,
-                                                     'password': None
                                                  },
                                                  'XYZ': {'567': 987}
                                              },
@@ -138,14 +140,28 @@ class TestConfiguration:
             'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
-                'username': None,
-                'password': None
             },
             'XYZ': {'567': 987}
         }
 
         configuration = Configuration.read()
         assert configuration == expected_configuration
+        version = get_api_version()
+        assert configuration.get_user_agent() == 'HDXPythonLibrary/%s-test' % version
+
+        Configuration._create(
+            hdx_config_dict={'hdx_site': 'prod', 'hdx_read_only': True, 'hdx_key': 'abcde', 'user_agent': 'test'},
+            hdx_base_config_dict={
+                'hdx_prod_site': {
+                    'url': 'https://data.humdata.org/',
+                },
+                'XYZ': {'567': 987}
+            },
+            project_config_yaml=project_config_yaml)
+        configuration = Configuration.read()
+        expected_configuration['user_agent'] = 'test'
+        assert configuration == expected_configuration
+        assert configuration.get_user_agent() == 'HDXPythonLibrary/%s-test' % version
 
         smtp_initargs = {
             'host': 'localhost',
@@ -202,8 +218,6 @@ hello there'''
             'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
-                'username': None,
-                'password': None
             },
             'hdx_test_site': {
                 'url': 'https://test-data.humdata.org/',
@@ -229,8 +243,6 @@ hello there'''
             'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
-                'username': None,
-                'password': None
             },
             'hdx_test_site': {
                 'url': 'https://test-data.humdata.org/',
@@ -255,23 +267,18 @@ hello there'''
             'hdx_key': '12345',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
-                'username': None,
-                'password': None
             },
             'hdx_demo_site': {
                 'url': 'https://demo-data.humdata.org/',
-                'username': 'ZGVtbzE3OQ==',
-                'password': 'ZnVud2l0aGhkeA=='
             },
             'hdx_test_site': {
                 'url': 'https://test-data.humdata.org/',
-                'username': 'ZGF0YXByb2plY3Q=',
-                'password': 'aHVtZGF0YQ=='
             },
             'hdx_feature_site': {
                 'url': 'https://feature-data.humdata.org/',
-                'username': 'ZGF0YXByb2plY3Q=',
-                'password': 'aHVtZGF0YQ=='
+            },
+            'hdx_dev_site': {
+                'url': 'http://data.dev.humdata.org/',
             },
             'dataset': {'required_fields': [
                 'name',
@@ -350,23 +357,18 @@ hello there'''
             'hdx_key': '12345',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
-                'username': None,
-                'password': None
             },
             'hdx_demo_site': {
                 'url': 'https://demo-data.humdata.org/',
-                'username': 'ZGVtbzE3OQ==',
-                'password': 'ZnVud2l0aGhkeA=='
             },
             'hdx_test_site': {
                 'url': 'https://test-data.humdata.org/',
-                'username': 'ZGF0YXByb2plY3Q=',
-                'password': 'aHVtZGF0YQ=='
             },
             'hdx_feature_site': {
                 'url': 'https://feature-data.humdata.org/',
-                'username': 'ZGF0YXByb2plY3Q=',
-                'password': 'aHVtZGF0YQ=='
+            },
+            'hdx_dev_site': {
+                'url': 'http://data.dev.humdata.org/',
             },
             'my_param': 'abc',
             'dataset': {'required_fields': [
@@ -444,23 +446,18 @@ hello there'''
             'tags_cleanup_url': 'https://raw.githubusercontent.com/OCHA-DAP/hdx-python-api/master/tests/fixtures/HDX_Tags_Cleaning.csv',
             'hdx_prod_site': {
                 'url': 'https://data.humdata.org/',
-                'username': None,
-                'password': None
             },
             'hdx_demo_site': {
                 'url': 'https://demo-data.humdata.org/',
-                'username': 'ZGVtbzE3OQ==',
-                'password': 'ZnVud2l0aGhkeA=='
             },
             'hdx_test_site': {
                 'url': 'https://test-data.humdata.org/',
-                'username': 'ZGF0YXByb2plY3Q=',
-                'password': 'aHVtZGF0YQ=='
             },
             'hdx_feature_site': {
                 'url': 'https://feature-data.humdata.org/',
-                'username': 'ZGF0YXByb2plY3Q=',
-                'password': 'aHVtZGF0YQ=='
+            },
+            'hdx_dev_site': {
+                'url': 'http://data.dev.humdata.org/',
             },
             'dataset': {'required_fields': [
                 'name',
@@ -533,7 +530,7 @@ hello there'''
         actual_configuration = Configuration.read()
         assert actual_configuration.get_api_key() == '12345'
         assert actual_configuration.get_hdx_site_url() == 'https://data.humdata.org/'
-        assert actual_configuration._get_credentials() == ('', '')
+        assert actual_configuration._get_credentials() is None
 
     def test_set_hdx_key_value(self, project_config_yaml):
         Configuration._create(user_agent='test', hdx_site='prod', hdx_key='TEST_HDX_KEY',
@@ -582,7 +579,8 @@ hello there'''
         assert Configuration.read().remoteckan() == remoteckan
         remoteckan = ckanapi.RemoteCKAN('http://hahaha', apikey='54321',
                                         user_agent='HDXPythonLibrary/0.5')
-        Configuration._create(remoteckan=remoteckan,
+        Configuration._create(user_agent='test',
+                              remoteckan=remoteckan,
                               hdx_site='prod', hdx_key='TEST_HDX_KEY',
                               hdx_base_config_dict={},
                               project_config_yaml=project_config_yaml)
@@ -598,7 +596,7 @@ hello there'''
                         empty_yaml, user_agent_config_wrong_yaml, project_config_yaml):
         Configuration._create(user_agent_config_yaml=user_agent_config_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                               hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
-        version = Configuration.get_version()
+        version = get_api_version()
         assert Configuration.read().remoteckan().user_agent == 'lala:HDXPythonLibrary/%s-myua' % version
         Configuration._create(user_agent_config_yaml=user_agent_config2_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                               hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
@@ -612,16 +610,17 @@ hello there'''
         Configuration._create(user_agent='my_ua', preprefix='papa', hdx_site='prod', hdx_key='TEST_HDX_KEY',
                               hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         assert Configuration.read().remoteckan().user_agent == 'papa:HDXPythonLibrary/%s-my_ua' % version
-        with pytest.raises(ConfigurationError):
+        with pytest.raises(UserAgentError):
             Configuration._create(user_agent_config_yaml=user_agent_config3_yaml, user_agent_lookup='fail',
                                   hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                   hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
         with pytest.raises(LoadError):
             Configuration._create(user_agent_config_yaml=empty_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                   hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
-        with pytest.raises(ConfigurationError):
+        with pytest.raises(UserAgentError):
             Configuration._create(user_agent_config_yaml=user_agent_config_wrong_yaml, hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                   hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
-        with pytest.raises(ConfigurationError):
+        with pytest.raises(UserAgentError):
             Configuration._create(hdx_site='prod', hdx_key='TEST_HDX_KEY',
                                   hdx_base_config_dict={}, project_config_yaml=project_config_yaml)
+
